/*
FwCalendar: Monthly Calendar for FiveWin.

Sintaxis: FwCalendar( [<dDate>] [,<nRow>, <nCol> [, <oWnd> ] ) --> dNewDate

Parmeters:

   <dDate>: Optional.- Date Field or variable. Defaults Date().

   <nRow>: Top pixel where calendar must be shown. If ommited, the Calendar
           will be shown Centered.

   <nCol>: Same for left pixel.

   <oWnd>: Reference of the container object. It can be a Dialog, a Window or
           aControl.

Returns:
   A date Variable selected in calendar.


Notes:
   You can get relative position for nRow and nCol with function
   GetCoors() as follows:

   aRect := GetCoors( oGet:hWnd )
   nRow := aRect[ 1 ]
   nCol := aRect[ 4 ]

Compile with:  /n /a /v

Author: Manuel Mercado,
        Freelance programmer.
        Salamanca, Gto., Mexico.

Version: 2.0  March 1st, 2004  FreeWare

   This version includes a second utility "Operations with dates", just
   right-click the calendar to use it.

   No longer a resource dialog is required, it's fully Clipper/FiveWin
   source code.

   Now is Multilingual, just link the object files: MsgXXX.obj and
   NtxXXX.obj with your app. ( eg. for Spanish MsgSpa.obj, NtxSpa.obj ), ;
   you'll find them at Clipper\Obj directory.

   For the "Date operations" dialog, you should translate the aTxt array to
   your own language (text strings are originally for English/Spanish).


Enjoy it

*/

#ifdef __CLIPPER__
   #define _OBJECTS_CH
   #define _DDE_CH
   #define _VIDEO_CH
   #define _TREE_CH
   #define _ODBC_CH
#endif

#include "FiveWin.ch"

#define WS_3DLOOK  4   // 0x4L

MEMVAR nYear, nMonth

Static dDate, dDate1, dDate2, dResult, nOper, cOper2, cOper3, lEng, aTxt[ 13 ]

//---------------------------------------------------------------------------------------------//

Function FwCalendar( dInDate, nRow, nCol, oWnd )

   Local nEle, nId, xVar, hBrush, cDateFormat, oRect, aFonts[ 2 ], ;
         lOk := .F.

   Private cMonth, oDlg, cDate, aMonth, aDay, aObj[ 7 ], nLast, aCoords, ;
           aSay[ 42 ], cSay[ 42 ], nYear, nMonth, nDay

   If ValType( dInDate ) != "D" .or. Empty( dInDate )
      dDate := Date()
   Else
      dDate := dInDate
   EndIf

   Default nLast   := 1, ;
           aCoords := {}, ;
           aSay    := Array( 37 ), ;
           cSay    := Array( 37 ), ;
           nYear   := Year( dDate ), ;
           nMonth  := Month ( dDate ), ;
           nDay    := Day( dDate ), ;
           nCol    := 0

   lEng := "JANUARY" $ Upper( cMonth( StoD( "19800101" ) ) )

// Translate the Non-English strings to your own language

   aTxt[ 1 ]  := If( lEng, "Elapsed days in this year: ", ;
                    "Das transcurridos en el ao: " )
   aTxt[ 2 ]  := If( lEng, "; Pending days: ", ;
                    "; Faltan por transcurrir: " )
   aTxt[ 3 ]  := If( lEng, "Date operations", "Operaciones con fechas" )
   aTxt[ 4 ]  := If( lEng, "Date:", "Fecha:" )
   aTxt[ 5 ]  := If( lEng, "Date", "Fecha" )
   aTxt[ 6 ]  := If( lEng, "Days", "Das" )
   aTxt[ 7 ]  := If( lEng, "Days:", "Das:" )
   aTxt[ 8 ]  := If( lEng, "Date/Days:", "Fecha/Das:" )
   aTxt[ 9 ]  := If( lEng, "Plus", "Ms" )
   aTxt[ 10 ] := If( lEng, "Minus", "Menos" )
   aTxt[ 11 ] := If( lEng, "SuMoTuWeThFrSa", "DoLuMaMiJuViSa" )
   aTxt[ 12 ] := If( lEng, "&Accept", "&Aceptar" )
   aTxt[ 13 ] := If( lEng, "&Exit", "&Salir" )

   aMonth := {}

   For nEle := 1 To 12
      AAdd( aMonth, cMonth( StoD( LTStr( Year( Date() ) ) + ;
                                  StrZero( nEle, 2 ) + "01" ) ) )
   Next

   nId := Date()

   While Dow( nId ) != 1
      nId++
   EndDo

   aDay := {}

   For nEle := 1 To 7
      AAdd( aDay, OemToAnsi( cDoW( nId++ ) ) )
   Next

   If ValType( oWnd ) = "O"

      oRect := oWnd:GetRect()

      If ValType( nRow ) = "N"
         nRow += oRect:nTop
         nCol += oRect:nLeft
      EndIf

      If ( nEle := ( GetSysMetrics( 1 ) - ( nRow + 254 ) ) ) < 0
         nRow += nEle
      EndIf

      If ( nEle := ( GetSysMetrics( 0 ) - ( nCol + 158 ) ) ) < 0
         nCol += nEle
      EndIf

   EndIf

   If lEng
      cDate := aDay[ Dow( dDate ) ] + ", " + aMonth[ Month( dDate ) ] + ;
               Space( 1 ) + LTrim( Str( Day( dDate ) ) )
   Else
      cDate := aDay[ Dow( dDate ) ] + ", " + LTrim( Str( Day( dDate ) ) ) + ;
               " de " + aMonth[ Month( dDate ) ]
   EndIf

   nMonth := Month( dDate )
   cMonth := aMonth[ nMonth ]

   DEFINE FONT aFonts[ 1 ] NAME "MS Sans Serif" SIZE 0, -11
   DEFINE FONT aFonts[ 2 ] NAME "MS Sans Serif" SIZE 0, -13

   DEFINE DIALOG oDlg FROM 0, 0 TO 216, 153 PIXEL ;
          COLORS CLR_BLACK, CLR_HGRAY ;
          STYLE nOr( DS_MODALFRAME, WS_POPUP, WS_VISIBLE, WS_DLGFRAME, ;
                     WS_SYSMENU, WS_3DLOOK )

   @  0,  5 SAY aObj[ 1 ] VAR cDate OF oDlg CENTER ;
            FONT aFonts[ 1 ] UPDATE ;
            COLORS CLR_BLACK, CLR_HGRAY SIZE 68, 8 PIXEL

   @  8,  5 COMBOBOX aObj[ 2 ] VAR cMonth OF oDlg ;
            ITEMS aMonth SIZE 40, 182 PIXEL ;
            COLORS CLR_BLACK, CLR_WHITE FONT aFonts[ 1 ] ;
            ON CHANGE ( nMonth := AScan( aMonth, cMonth ), ;
                        fActMonth( aSay, cSay ) )

   @  8, 47 GET aObj[ 3 ] VAR nYear OF oDlg PICTURE "@K 9999" ;
            FONT aFonts[ 1 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 17, 10 PIXEL ;
            SPINNER ON UP ( ++nYear, aObj[ 3 ]:Refresh(), ;
                            fActMonth( aSay, cSay ), aObj[ 5 ]:SetFocus() ) ;
            ON DOWN ( --nYear, aObj[ 3 ]:Refresh(), fActMonth( aSay, cSay ), ;
                     aObj[ 5 ]:SetFocus() )

   aObj[ 3 ]:bGotFocus := { |x| x:SetColor( CLR_WHITE, CLR_BLUE ) }
   aObj[ 3 ]:bLostFocus := { || aObj[ 3 ]:SetColor( CLR_BLACK, CLR_WHITE ), ;
                                fActMonth( aSay, cSay ), ;
                                aObj[ 5 ] :SetFocus() }

   @110,  0 LISTBOX aObj[ 5 ] VAR xVar ITEMS {} OF oDlg SIZE 65, 19 PIXEL

   aObj[ 5 ]:bKeyDown := { |nkey| fSetDay( nKey ) }
   aObj[ 5 ]:bLostFocus := { || aSay[ nLast ]:SetColor( CLR_WHITE, ;
                                CLR_GRAY ), aSay[ nLast ]:Refresh() }
   aObj[ 5 ]:bGotFocus := { || fActMonth( aSay, cSay ) }

   @ 20,  5 SAY aObj[ 4 ] PROMPT "" OF oDlg ;
            FONT aFonts[ 1 ] UPDATE ;
            COLORS CLR_WHITE, CLR_BLUE SIZE 68, 8 PIXEL

   @ 20,  5 SAY SubStr( aTxt[ 11 ], 1, 2 ) OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_WHITE, CLR_BLUE SIZE 8, 8 PIXEL

   @ 20, 15 SAY SubStr( aTxt[ 11 ], 3, 2 ) OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_WHITE, CLR_BLUE SIZE 8, 8 PIXEL

   @ 20, 25 SAY SubStr( aTxt[ 11 ], 5, 2 ) OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_WHITE, CLR_BLUE SIZE 8, 8 PIXEL

   @ 20, 35 SAY SubStr( aTxt[ 11 ], 7, 2 ) OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_WHITE, CLR_BLUE SIZE 8, 8 PIXEL

   @ 20, 45 SAY SubStr( aTxt[ 11 ], 9, 2 ) OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_WHITE, CLR_BLUE SIZE 8, 8 PIXEL

   @ 20, 55 SAY SubStr( aTxt[ 11 ], 11, 2 ) OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_WHITE, CLR_BLUE SIZE 8, 8 PIXEL

   @ 20, 65 SAY SubStr( aTxt[ 11 ], 13, 2 ) OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_WHITE, CLR_BLUE SIZE 8, 8 PIXEL

   @ 29,  5 SAY aSay[ 1 ] VAR cSay[ 1 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 29, 15 SAY aSay[ 2 ] VAR cSay[ 2 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 29, 25 SAY aSay[ 3 ] VAR cSay[ 3 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 29, 35 SAY aSay[ 4 ] VAR cSay[ 4 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 29, 45 SAY aSay[ 5 ] VAR cSay[ 5 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 29, 55 SAY aSay[ 6 ] VAR cSay[ 6 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 29, 65 SAY aSay[ 7 ] VAR cSay[ 7 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 39,  5 SAY aSay[ 8 ] VAR cSay[ 8 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 39, 15 SAY aSay[ 9 ] VAR cSay[ 9 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 39, 25 SAY aSay[ 10 ] VAR cSay[ 10 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 39, 35 SAY aSay[ 11 ] VAR cSay[ 11 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 39, 45 SAY aSay[ 12 ] VAR cSay[ 12 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 39, 55 SAY aSay[ 13 ] VAR cSay[ 13 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 39, 65 SAY aSay[ 14 ] VAR cSay[ 14 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 49,  5 SAY aSay[ 15 ] VAR cSay[ 15 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 49, 15 SAY aSay[ 16 ] VAR cSay[ 16 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 49, 25 SAY aSay[ 17 ] VAR cSay[ 17 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 49, 35 SAY aSay[ 18 ] VAR cSay[ 18 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 49, 45 SAY aSay[ 19 ] VAR cSay[ 19 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 49, 55 SAY aSay[ 20 ] VAR cSay[ 20 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 49, 65 SAY aSay[ 21 ] VAR cSay[ 21 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 59,  5 SAY aSay[ 22 ] VAR cSay[ 22 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 59, 15 SAY aSay[ 23 ] VAR cSay[ 23 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 59, 25 SAY aSay[ 24 ] VAR cSay[ 24 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 59, 35 SAY aSay[ 25 ] VAR cSay[ 25 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 59, 45 SAY aSay[ 26 ] VAR cSay[ 26 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 59, 55 SAY aSay[ 27 ] VAR cSay[ 27 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 59, 65 SAY aSay[ 28 ] VAR cSay[ 28 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 69,  5 SAY aSay[ 29 ] VAR cSay[ 29 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 69, 15 SAY aSay[ 30 ] VAR cSay[ 30 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 69, 25 SAY aSay[ 31 ] VAR cSay[ 31 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 69, 35 SAY aSay[ 32 ] VAR cSay[ 32 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 69, 45 SAY aSay[ 33 ] VAR cSay[ 33 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 69, 55 SAY aSay[ 34 ] VAR cSay[ 34 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 69, 65 SAY aSay[ 35 ] VAR cSay[ 35 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 79,  5 SAY aSay[ 36 ] VAR cSay[ 36 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 79, 15 SAY aSay[ 37 ] VAR cSay[ 37 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 79, 25 SAY aSay[ 38 ] VAR cSay[ 38 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 79, 35 SAY aSay[ 39 ] VAR cSay[ 39 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 79, 45 SAY aSay[ 40 ] VAR cSay[ 40 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 79, 55 SAY aSay[ 41 ] VAR cSay[ 41 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 79, 65 SAY aSay[ 42 ] VAR cSay[ 42 ] OF oDlg ;
            FONT aFonts[ 2 ] UPDATE ;
            COLORS CLR_BLACK, CLR_WHITE SIZE 8, 8 PIXEL

   @ 91,  5 BUTTON aObj[ 6 ] PROMPT aTxt[ 12 ] OF oDlg ;
            ACTION ( lOk := .T., oDlg:End() ) ;
            FONT aFonts[ 1 ] SIZE 29, 12 PIXEL UPDATE

   @ 91, 43 BUTTON aObj[ 7 ] PROMPT aTxt[ 13 ] OF oDlg ACTION oDlg:End() ;
            FONT aFonts[ 1 ] SIZE 29, 12 PIXEL UPDATE

   oDlg:bLClicked := { |nRow, nCol| aObj[ 5 ]:SetFocus(), fSetDay( nRow, nCol ) }
   oDlg:bGotFocus := { || aObj[ 5 ]:SetFocus() }
   oDlg:bRClicked := { || fDateOper( dDate ) }

   ACTIVATE DIALOG oDlg ;
            ON INIT ( If( ValType( nRow ) != "N", WndCenter( oDlg:hWnd ), ;
                          oDlg:Move( nRow, nCol ) ), ;
                      AEval( aSay, { | e,n | ;
                      AAdd( aCoords, GetCoors( aSay[ n ]:hWnd ) ) } ), ;
                      fActMonth( aSay, cSay ), aObj[ 5 ]:SetFocus() ) ;
            ON PAINT ( hBrush := CreateSolidBrush( CLR_WHITE ), ;
                       FillRect( oDlg:hDC, ;
                                 TRect():New( aCoords[ 1, 1 ], ;
                                 aCoords[ 1, 2 ], aCoords[ 42, 3 ], ;
                                 aCoords[ 42, 4 ] ), hBrush ), ;
                       DeleteObject( hBrush ), ;
                      oDlg:Box( aCoords[ 1, 1 ] - 1, aCoords[ 1, 2 ] - 1, ;
                                aCoords[ 42, 3 ], aCoords[ 42, 4 ] ) )

   If lOk
      dDate := StoD( LTStr( nYear )  + StrZero( nMonth, 2 ) + ;
                     StrZero( nDay, 2 ) )
   EndIf

   If oRect != Nil
      DeleteObject( oRect )
   EndIf

   Aeval( aFonts, { |o| o:End() } )

Return dDate

//--------------------------------------------------------------------------//

Static Function fActMonth( aSay, cSay )

   Local lActive, nEle, ;
         dNewDate := StoD( Str( nYear, 4 ) + StrZero( nMonth, 2 ) + "01" ), ;
         nNewDay := 1

   Local nLastDay := dNewDate + 32

   MemVar aDate, aMonth, nDay, nMonth, nYear, aDay, nLast, aObj, cDate, ;
          aCoords

   If aDay == Nil
      Return Nil
   EndIf

   nLastDay := nLastDay - Day( nLastDay )
   nLastDay := Day( nLastDay )
   nDay := If( nDay > nLastDay, nLastDay, nDay )
   AFill( cSay, Space( 2 ) )
   AEval( aSay, { |e,n| aSay[ n ]:Refresh() } )
   nEle    := Dow( dNewDate )
   lActive := aObj[ 5 ]:lFocused

   While Month( dNewDate ) == nMonth

      cSay[ nEle ] := Str( nNewDay++, 2 )

      If nDay == Day( dNewDate )
         aSay[ nLast ]:SetColor( CLR_BLACK, CLR_WHITE )
         aSay[ nLast ]:Refresh()
         aSay[ nEle ]:SetColor( CLR_WHITE, If( lActive, CLR_BLUE, CLR_GRAY ) )
         aSay[ nEle ]:Refresh()
         nLast := nEle
      Else
         aSay[ nEle ]:SetColor( CLR_BLACK, CLR_WHITE )
         aSay[ nEle ]:Refresh()
      EndIf

      ++nEle
      ++dNewDate

   EndDo

   dNewDate := StoD( LTStr( nYear ) + StrZero( nMonth, 2 ) + ;
               StrZero( nDay, 2 ) )

   If lEng
      cDate := aDay[ Dow( dNewDate ) ] + ", " + ;
               aMonth[ Month( dNewDate ) ] + Space( 1 ) + ;
               LTrim( Str( Day( dNewDate ) ) )
   Else
      cDate := aDay[ Dow( dNewDate ) ] + ", " + ;
               LTrim( Str( Day( dNewDate ) ) ) + " de " + ;
               aMonth[ Month( dNewDate ) ]
   EndIf

   aObj[ 1 ]:Refresh()
   dDate := dNewDate

Return Nil

//--------------------------------------------------------------------------//

Static Function fSetDay( nRow, nCol )

   Local dNewDate, ;
         nEle := 0

   MemVar aSay, nLast, cSay, aMonth, nDay, nMonth, nYear, cDate, aCoords, aDay, aObj

   If ValType( nCol ) = "N"
      AEval( aCoords, { |x,n| nEle := If( nRow >= x[ 1 ] .and. ;
                                          nCol >= x[ 2 ] .and. ;
                                          nRow <= x[ 3 ] .and. ;
                                          nCol <= x[ 4 ], n, nEle ) } )

      If nEle > 0 .and. nEle <= 37 .and. Val( cSay[ nEle ] ) > 0
         aSay[ nLast ]:Setcolor( CLR_BLACK, CLR_WHITE )
         aSay[ nLast ]:Refresh()
         aSay[ nEle ]:Setcolor( CLR_WHITE, CLR_BLUE )
         aSay[ nEle ]:Refresh()
         nLast := nEle
         nDay  := Val(cSay[ nEle ] )
      EndIf

   Else

      Do Case

         Case nRow = VK_LEFT

            If nLast = 1 .or. Empty( cSay[ nLast - 1 ] )
               Return Nil
            EndIf

            aSay[ nLast ]:Setcolor( CLR_BLACK, CLR_WHITE )
            aSay[ nLast ]:Refresh()
            aSay[ nLast - 1 ]:Setcolor( CLR_WHITE, CLR_BLUE )
            aSay[ nLast - 1 ]:Refresh()
            nDay := Val( cSay[ --nLast ] )

         Case nRow = VK_RIGHT

            If nLast = 37 .or. Empty( cSay[ nLast + 1 ] )
               Return Nil
            EndIf

            aSay[ nLast ]:Setcolor( CLR_BLACK, CLR_WHITE )
            aSay[ nLast ]:Refresh()
            aSay[ nLast + 1 ]:Setcolor( CLR_WHITE, CLR_BLUE )
            aSay[ nLast + 1 ]:Refresh()
            nDay := Val( cSay[ ++nLast ] )

         Case nRow = VK_UP

            If nLast < 8 .or. Empty( cSay[ nLast - 7 ] )
               Return Nil
            EndIf

            aSay[ nLast ]:Setcolor( CLR_BLACK, CLR_WHITE )
            aSay[ nLast ]:Refresh()
            aSay[ nLast - 7 ]:Setcolor( CLR_WHITE, CLR_BLUE )
            aSay[ nLast - 7 ]:Refresh()
            nDay := Val( cSay[ nLast -= 7 ] )

         Case nRow = VK_DOWN

            If nLast > 35 .or. ( nLast + 7 ) > 37 .or. ;
               Empty( cSay[ nLast + 7 ] )
               Return Nil
            EndIf

            aSay[ nLast ]:Setcolor( CLR_BLACK, CLR_WHITE )
            aSay[ nLast ]:Refresh()
            aSay[ nLast + 7 ]:Setcolor( CLR_WHITE, CLR_BLUE )
            aSay[ nLast + 7 ]:Refresh()
            nDay := Val( cSay[ nLast += 7 ] )

      EndCase

   EndIf

   dNewDate := StoD( LTStr( nYear ) + StrZero( nMonth, 2 ) + ;
               StrZero( nDay, 2 ) )

   If lEng
      cDate := aDay[ Dow( dNewDate ) ] + ", " + ;
               aMonth[ Month( dNewDate ) ] + Space( 1 ) + ;
               LTrim( Str( Day( dNewDate ) ) )
   Else
      cDate := aDay[ Dow( dNewDate ) ] + ", " + ;
               LTrim( Str( Day( dNewDate ) ) ) + " de " + ;
               aMonth[ Month( dNewDate ) ]
   EndIf

   aObj[ 1 ]:Refresh()

Return Nil

//---------------------------------------------------------------------------------------------//

Static Function fDateOper()

   Local oDlg, aCtl[ 7 ], oFont, cElapDays, ;
         lCent := __SetCentury( .T. )

   MemVar cDate

   Default dDate1  := PadR( DtoC( dDate ), 11 ), ;
           nOper   := 1, ;
           dDate2  := Space( 11 ), ;
           dResult := DtoC( dDate ), ;
           cOper2  := aTxt[ 6 ], ;
           cOper3  := aTxt[ 5 ]

   cElapDays := aTxt[ 1 ] + LTStr( dDate - StoD( LTStr( Year( dDate ) ) + ;
                "0101" ) + 1 ) + aTxt[ 2 ] + ;
                LTStr( ( StoD( LTStr( Year( dDate ) + 1 ) + "0101" ) - ;
                         dDate ) - 1 )

   DEFINE FONT oFont NAME "MS Sans Serif" SIZE 0, -11

   DEFINE DIALOG oDlg FROM 0, 0 TO 146, 322 PIXEL ;
          COLORS CLR_BLACK, CLR_HGRAY TITLE aTxt[ 3 ] ;
          STYLE nOR( DS_MODALFRAME, WS_POPUP, WS_CAPTION, WS_SYSMENU, 4 )

   @  5,  5 SAY cDate OF oDlg ;
            FONT oFont UPDATE ;
            COLORS CLR_BLACK, CLR_HGRAY SIZE 200, 10 PIXEL

   @ 13,  5 SAY cElapDays OF oDlg ;
            FONT oFont UPDATE ;
            COLORS CLR_BLACK, CLR_HGRAY SIZE 200, 10 PIXEL

   @ 31,  5 SAY aCtl[ 1 ] PROMPT aTxt[ 4 ] OF oDlg ;
            FONT oFont UPDATE ;
            COLORS CLR_BLACK, CLR_HGRAY SIZE 18, 10 PIXEL

   @ 31, 24 GET aCtl[ 2 ] VAR dDate1 OF oDlg ;
            FONT oFont UPDATE PICTURE "@K!" ;
            VALID ( dResult := ;
                    If( nOper == 1, ;
                    If( Empty( dDate2 ) .or. IsNum( dDate2 ), ;
                        DtoC( CtoD( dDate1 ) + Val( dDate2 ) ), ;
                        DtoC( CtoD( dDate1 ) ) ), ;
                    If( Empty( dDate2 ) .or. IsNum( dDate2 ), ;
                        DtoC( CtoD( dDate1 ) - Val( dDate2 ) ), ;
                        LTStr( CtoD( dDate1 ) - CtoD( dDate2 ) ) ) ), ;
                        cOper2 := If( nOper == 1, aTxt[ 7 ], aTxt[ 8 ] ), ;
                        cOper3 := If( nOper == 1, aTxt[ 4 ], aTxt[ 7 ] ), ;
                        aCtl[ 4 ]:Refresh(), aCtl[ 6 ]:Refresh(), ;
                        aCtl[ 7 ]:Refresh(), .T. ) ;
            COLORS CLR_BLUE, CLR_WHITE SIZE 33, 10 PIXEL

   aCtl[ 2 ]:bGotFocus := { || aCtl[ 2 ]:SetColor( CLR_WHITE, CLR_BLUE ), ;
                               aCtl[ 2 ]:SelectAll(), ;
                               aCtl[ 2 ]:SetPos( 0 ), ;
                               dDate2 := PadR( If( Empty( dDate2 ) .or. ;
                               IsNum( dDate2 ), dDate2, ;
                               DtoC( CtoD( dDate2 ) ) ), 11 ), ;
                               aCtl[ 5 ]:Refresh() }
   aCtl[ 2 ]:bLostFocus := { || aCtl[ 2 ]:SetColor( CLR_BLUE, CLR_WHITE ) }

   @ 31, 59 COMBOBOX aCtl[ 3 ] VAR nOper OF oDlg ;
            ITEMS { aTxt[ 9 ], aTxt[ 10 ] } SIZE 29, 52 PIXEL ;
            COLORS CLR_BLUE, CLR_WHITE FONT oFont ;
            ON CHANGE ( dResult := ;
                        If( nOper == 1, ;
                        If( Empty( dDate2 ) .or. IsNum( dDate2 ), ;
                            DtoC( CtoD( dDate1 ) + Val( dDate2 ) ), ;
                            DtoC( CtoD( dDate1 ) ) ), ;
                        If( Empty( dDate2 ) .or. IsNum( dDate2 ), ;
                            DtoC( CtoD( dDate1 ) - Val( dDate2 ) ), ;
                            LTStr( CtoD( dDate1 ) - CtoD( dDate2 ) ) ) ), ;
                        cOper2 := If( nOper == 1, aTxt[ 7 ], aTxt[ 8 ] ), ;
                        cOper3 := If( nOper == 1, aTxt[ 4 ], ;
                        If( Empty( dDate2 ) .or. IsNum( dDate2 ), ;
                            aTxt[ 4 ], aTxt[ 7 ] ) ), ;
                        aCtl[ 4 ]:Refresh(), aCtl[ 6 ]:Refresh(), ;
                        aCtl[ 7 ]:Refresh() )

   aCtl[ 3 ]:bGotFocus := {|| dDate1 := PadR( DtoC( CtoD( dDate1 ) ), 11 ), ;
                              aCtl[ 2 ]:Refresh() }

   @ 31, 91 SAY aCtl[ 4 ] VAR cOper2 OF oDlg ;
            FONT oFont RIGHT UPDATE ;
            COLORS CLR_BLACK, CLR_HGRAY SIZE 30, 10 PIXEL

   @ 31,124 GET aCtl[ 5 ] VAR dDate2 OF oDlg ;
            FONT oFont UPDATE PICTURE "@K!" ;
            COLORS CLR_BLUE, CLR_WHITE SIZE 33, 10 PIXEL ;
            VALID ( dResult := ;
                    If( nOper == 1, ;
                    If( Empty( dDate2 ) .or. IsNum( dDate2 ), ;
                        DtoC( CtoD( dDate1 ) + Val( dDate2 ) ), ;
                        DtoC( CtoD( dDate1 ) ) ), ;
                    If( Empty( dDate2 ) .or. IsNum( dDate2 ), ;
                        DtoC( CtoD( dDate1 ) - Val( dDate2 ) ), ;
                        LTStr( CtoD( dDate1 ) - CtoD( dDate2 ) ) ) ), ;
                        cOper2 := If( nOper == 1, aTxt[ 7 ], aTxt[ 8 ] ), ;
                        cOper3 := If( nOper == 1, aTxt[ 4 ], aTxt[ 7 ] ), ;
                        aCtl[ 4 ]:Refresh(), aCtl[ 6 ]:Refresh(), ;
                        aCtl[ 7 ]:Refresh(), .T. )

   aCtl[ 5 ]:bGotFocus := { || aCtl[ 5 ]:SetColor( CLR_WHITE, CLR_BLUE ), ;
                               aCtl[ 5 ]:Selectall(), aCtl[ 5 ]:SetPos( 0 ) }

   aCtl[ 5 ]:bLostFocus := { || aCtl[ 5 ]:SetColor( CLR_BLUE, CLR_WHITE ) }

   @ 52, 47 SAY aCtl[ 6 ] VAR cOper3 OF oDlg ;
            FONT oFont RIGHT UPDATE ;
            COLORS CLR_BLACK, CLR_HGRAY SIZE 30, 10 PIXEL

   @ 52, 80 GET aCtl[ 7 ] VAR dResult OF oDlg ;
            FONT oFont UPDATE MULTILINE NO VSCROLL READONLY ;
            COLORS CLR_BLUE, CLR_WHITE SIZE 31, 10 PIXEL

   aCtl[ 7 ]:nStyle := nAnd( aCtl[ 7 ]:nStyle, nNot( WS_TABSTOP ) )

   ACTIVATE DIALOG oDlg CENTERED  ;
            VALID ( oFont:End(), .T. )

   __SetCentury( lCent )

Return Nil

//---------------------------------------------------------------------------------------------//
// typing savings :)

Static Function LTStr( nNumber ) ; Return Ltrim( Str( nNumber ) )

//--------------------------------------------------------------------------//

Function IsNum( cExp )

   Local xVar01, xVar02, xVar03, xVar04

   If Empty( cExp ) .or. ValType( cExp ) != "C"
      Return .F.
   EndIf

   cExp := AllTrim( cExp )
   xVar04 := xVar03 := 0

   For xVar01 := 1 To Len( cExp )
      xVar02 := SubStr( cExp, xVar01, 1 )

      If ! IsDigit( xVar02 ) .and. ! ( xVar02 $ ".+-" )
         Return .F.
      EndIf

      If xVar02 $ "+-"
         xVar03 += If( xVar01 == 1, 1, 2 )
      ElseIf xVar02 = "."
         ++xVar04
      EndIf

   Next

Return If( xVar04 > 1 .or. xVar03 > 1, .F., .T. )

//--------------------------------------------------------------------------//

Function StoD( cAnsiDate )

   Local cFormat := Upper( Set( _SET_DATEFORMAT ) ), ;
         cYear   := SubStr( cAnsiDate, 1, 4 ), ;
         cMon    := SubStr( cAnsiDate, 5, 2 ), ;
         cDay    := SubStr( cAnsiDate, 7, 2 )

   cFormat := StrTran( cFormat, "YYYY", cYear )
   cFormat := StrTran( cFormat, "YY", Right( cYear, 2 ) )
   cFormat := StrTran( cFormat, "MM", cMon )
   cFormat := StrTran( cFormat, "DD", cDay )

Return CtoD( cFormat )

