* Class TSPoint: Specially usefull on passing parameters from cursor movement
*                or screen coordinates
*
* Example: pt := TSPoint():New( GetCursorPos() )
* ? pt:Row   // Result: Vertical pixel
* ? pt:Col   // Result: Horizontal pixel
* ? pt:y     // Result: Vertical pixel
* ? pt:x     // Result: Horizontal pixel
*
* Author Manuel Mercado

#include "FiveWin.ch"

//----------------------------------------------------------------------------//

CLASS TSPoint

   DATA   y, Row  // nRow
   DATA   x, Col  // nCol
   DATA   yp      // prior nRow
   DATA   xp      // prior nCol
   DATA   ym      // nRow pixels moved from prior nRow, negative means up movement
   DATA   xm      // nCol pixels moved from prior nCol, negative means left movement

   METHOD New( nRow, nCol ) CONSTRUCTOR
   METHOD Move( nRow, nCol )
   METHOD Reset( nRow, nCol )
   METHOD aGet( nCoor )
   METHOD nGet( nCoor )
   METHOD sGet( nCoor )

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New( uRow, nCol ) CLASS TSPoint

   Default uRow := 0

   If ValType( uRow ) == "A"

      If ValType( nCol ) == "L" .and. nCol // invert coors
         nCol := uRow[ 1 ]
         uRow := uRow[ 2 ]
      Else
         nCol := uRow[ 2 ]
         uRow := uRow[ 1 ]
      EndIf

   ElseIf Valtype( uRow ) == "N" .and. nCol == Nil
      nCol := nHiWord( uRow )
      uRow := nLoWord( uRow )
   EndIf

   ::x   = nCol
   ::Col = nCol
   ::y   = uRow
   ::Row = uRow
   ::xp  = nCol
   ::yp  = uRow
   ::xm  = 0
   ::ym  = 0

Return Self

//----------------------------------------------------------------------------//

METHOD Move( uRow, nCol ) CLASS TSPoint

   If ValType( uRow ) == "A"

      If ValType( nCol ) == "L" .and. nCol // invert coors
         nCol := uRow[ 1 ]
         uRow := uRow[ 2 ]
      Else
         nCol := uRow[ 2 ]
         uRow := uRow[ 1 ]
      EndIf

   ElseIf Valtype( uRow ) == "N" .and. nCol == Nil
      nCol := nHiWord( uRow )
      uRow := nLoWord( uRow )
   EndIf

   ::xp  = ::x
   ::yp  = ::y
   ::x   = nCol
   ::Col = nCol
   ::y   = uRow
   ::Row = uRow
   ::xm  = ::x - ::xp
   ::ym  = ::y - ::yp

Return Self

//----------------------------------------------------------------------------//

METHOD Reset( uRow, nCol ) CLASS TSPoint

   Default uRow := 0

   If ValType( uRow ) == "A"

      If ValType( nCol ) == "L" .and. nCol // invert coors
         nCol := uRow[ 1 ]
         uRow := uRow[ 2 ]
      Else
         nCol := uRow[ 2 ]
         uRow := uRow[ 1 ]
      EndIf

   ElseIf Valtype( uRow ) == "N" .and. nCol == Nil
      nCol := nHiWord( uRow )
      uRow := nLoWord( uRow )
   EndIf

   ::x   = nCol
   ::Col = nCol
   ::y   = uRow
   ::Row = uRow
   ::xp  = nCol
   ::yp  = uRow
   ::xm  = 0
   ::ym  = 0

Return Self

//----------------------------------------------------------------------------//
* METHOD TSPoint:aGet() Returns an array with:
* If nCoor = 1, Current coordinates (row, col)
* If nCoor = 2, Prior coordinates
* If nCoor = 3  Vertical, Horizontal Movement
* Defaults to 1
//----------------------------------------------------------------------------//

METHOD aGet( nCoor ) CLASS TSPoint

   Local aRet

   Default nCoor := 1

   aRet := If( nCoor < 2, { ::y, ::x }, If( nCoor == 2, { ::yp, ::xp }, { ::ym, ::xm } ) )

Return aRet

//----------------------------------------------------------------------------//
* METHOD TSPoint:nGet() Returns a Long Number with:
* If nCoor = 1, Current coordinates (row = nLoWord, col = nHiWord)
* If nCoor = 2, Prior coordinates
* If nCoor = 3  Vertical, Horizontal Movement
* Defaults to 1
//----------------------------------------------------------------------------//

METHOD nGet( nCoor ) CLASS TSPoint

   Local nRet

   Default nCoor := 1

   nRet := If( nCoor < 2, nMakeLong( ::y, ::x ), ;
               If( nCoor == 2, nMakeLong( ::yp, ::xp ), nMakeLong( ::ym, ::xm ) ) )

Return nRet

//----------------------------------------------------------------------------//
* METHOD TSPoint:sGet() Returns a Point Structure with:
* If nCoor = 1, Current coordinates (x, y)
* If nCoor = 2, Prior coordinates
* Defaults to 1
//----------------------------------------------------------------------------//

METHOD sGet( nCoor ) CLASS TSPoint

   Local sRet

   Default nCoor := 1

   sRet := If( nCoor < 2, I2Bin( ::x ) + I2Bin( ::y ), I2Bin( ::xp ) + I2Bin( ::yp ) )

Return sRet

//----------------------------------------------------------------------------//

Function pCl2Scr( hWnd, aPoint )

   If ValType( aPoint ) == "O"  // TSPoint object
      aPoint := aPoint:aGet()
   EndIf

Return TSPoint():New( ClientToScreen( hWnd, aPoint ) )

//----------------------------------------------------------------------------//

Function pScr2Cl( hWnd, aPoint )

   If ValType( aPoint ) == "O"  // TSPoint object
      aPoint := aPoint:aGet()
   EndIf

Return TSPoint():New( ScreenToClient( hWnd, aPoint ) )

//----------------------------------------------------------------------------//

Function pWndPos( hWnd )

   Local aRct := GetWndRect( hWnd )

Return TSPoint():New( aRct[ 1 ], aRct[ 2 ] )  // p:y = nRow, p:x = nCol

//----------------------------------------------------------------------------//

Function pWndSize( hWnd )

   Local rct := GetClientRect( hWnd )

Return TSPoint():New( rct[ 3 ], rct[ 4 ] )  // p:y = nHeight, p:x = nWidth
