* ============================================================================
* Debug.prg Version 1.0 Jul/15/2004
* Author: Manuel Mercado
*
* A little general purpose debugging tool that doesn't affect
* graphical events. To make it work, call the function SetDebug() from your
* FiveWin program.
*
* Syntax: SetDebug( [<lDebug>], [<cDebugFile>], [<lClear>], [<uCtrlKey>] )
*
* Parameter     Default                Meaning
* <lDebug>      .T.             Activates/Deactivates debugging.
* <cDebugFile>  ".\Debug.log"   Disk file full path to store the fDebug() results.
*                                  If not exists, it will be created.
* <lClear>      .T.             Clear the debug.log file.
* <uCtrlKey>    "D"             Letter or Ascii value which combined with
*                                  Ctrl Key activates the debug.log runtime viewer.
*
*
* fDebug() Syntax: fDebug( [<uMsg>], [<aVarNames>], [<aVarVal>], [<nActiv>] )
*
* Parameter     Default                Meaning
* <uMsg>        Null     String/Variable/Expression/Value to show.
* <aVarNames>   Nil      Array or list string of variables names to be analyzed
*                           eg. {"nImport","cName"} or "nImport,cName".
* <aVarVal>     Nil      Array of values when aVarNames variables are locals
*                           eg. {nImport,cName}.
* <nActiv>      1        Activation of ProcName/ProcLine.
*                        Use 2 or greater to know where the
*                        debugged procedure was called from
*
* When aVarNames or aVarVal or both are present, the debug screen will look like this:
*
* Procedure     Line   Text
* MYFUNCTION     325   umsg
*                      nImport==15.00
*                      cName==John Smith
*
* Compile with /N and Link it to your application.
* ============================================================================

#define _DDE_CH
#define _FOLDER_CH
#define _VIDEO_CH
#define _TREE_CH
#define _ODBC_CH
#define _OBJECTS_CH

#include "FiveWin.ch"
#include "\TSBrowse\Include\TSBrowse.ch"

#define END_PROC_NAME  Chr(5)+Chr(32)
#define END_PROC_LINE  Chr(7)+Chr(58)+Chr(32)

// my typing savings :-)
#xtranslate VtoC( <uVar> )  => cValToChar( <uVar> )
#xtranslate LTStr( <uVar> ) => LTrim( Str( <uVar> ) )

Static lMyDebug, cMyDebugFile, nMyHotKey

//--------------------------------------------------------------------------//

Function fDebug( uMsg, cVarList, aVarVal, nActiv )

   Local nHand, cLine, lGoDebug, cName, aVars, nEle, cVar, uVal, bError, oErr

   Default uMsg     := "", ;
           lMyDebug := .F., ;
           nActiv   := 1

   If ValType( lMyDebug ) != "L" .or. ! lMyDebug
      Return Nil
   EndIf

   cName := If( cMyDebugFile == Nil .or. Empty( cMyDebugFile ), ".\Debug.log", ;
                cMyDebugFile )

   uMsg := VtoC( uMsg )
   nHand := FOpen( cName, 2 )

   If nHand < 0
      MsgStop( "Sorry, Can't open Debug file", "Error " + VtoC( FError() ) )
      Return Nil
   EndIf

   cLine := ""

   For nEle := nActiv To 1 Step -1
      cLine += ProcName( nEle ) + ;
               If( nEle > 1, "(" + LTStr( ProcLine( nEle ) ) + ")->", "" )
   Next

   cLine += END_PROC_NAME + LTStr( ProcLine( 1 ) ) + END_PROC_LINE + ;
            uMsg + CRLF

   If cVarList != Nil .and. ! Empty( cVarList )

      If ValType( cVarList ) == "A"
         aVars := cVarList
      Else
         aVars := CtoA( cVarList, "C", "," )
      EndIf

      If aVarVal == Nil // the list has names of visible variables
                        // ( static, private or public )
                        // then, the array of values is not necessary

         bError := ErrorBlock( { | oErr | Break( oErr ) } )

         For nEle := 1 To Len( aVars )

            Begin Sequence
               cVar := aVars[ nEle ]
               uVal := &cVar
               cLine += cVar + "==" + VtoC( uVal ) + CRLF
            Recover Using oErr
               cLine += aVars[ nEle ] + " Error (" + VtoC( oErr:genCode ) + ;
                        ") " + If( ! Empty( VtoC( oErr:Description ) ), ;
                        oErr:Description, "Not Defined" ) + CRLF
            End Sequence

         Next

         ErrorBlock( bError )

      Else  // local vars values in aVarVal

         ASize( aVarVal, Len( aVars ) )

         For nEle := 1 To Len( aVars )
            cLine += aVars[ nEle ] + "==" + VtoC( aVarVal[ nEle ] ) + CRLF
         Next

      EndIf

   EndIf

   FSeek( nHand, 0, 2 )
   FWrite( nHand, cLine )
   FClose( nHand )

Return Nil

//--------------------------------------------------------------------------//

Function SetDebug( lDeb, cDbFile, lClear, uCtrlKey )

   Local nHand

   Default lDeb     := .T., ;
           cDbFile  := If( cMyDebugfile == Nil, ".\Debug.log", cMyDebugFile ), ;
           lClear   := .T.
           uCtrlKey := If( nMyHotKey == Nil, "D", nMyHotkey )

   uCtrlKey := If( ValType( uCtrlKey ) == "C", Asc( uCtrlKey ), uCtrlKey )

   If lDeb

      If lClear .or. ! File( cDbFile )

         nHand := FCreate( cDbFile, 0 )

         If nHand < 0
            MsgStop( "Sorry, Can't create Debug file", "Error " + VtoC( FError() ) )
            Return Nil
         EndIf

         FClose( nHand )

      EndIf

      If uCtrlKey != Nil
         SetKey( nMyHotKey := uCtrlKey, {|| If( GetKeyState( VK_CONTROL ), ;
                 ( lMyDebug := .F., fShowDebug(), lMyDebug := .T. ), Nil ) } )
      EndIf

      lMyDebug     := lDeb
      cMyDebugFile := cDbFile

   Else

      lMyDebug     := Nil
      cMyDebugFile := Nil

      If nMyHotKey != Nil
         SetKey( nMyHotKey, Nil )
      EndIf

      nMyHotKey := Nil

   EndIf

Return Nil

//--------------------------------------------------------------------------//
// creates an array from a string with or without separators
// It's part of my personal Udf's collection.

Static Function CtoA( cString, cType, cSeparator, bBlock )

   Local cToken, nEle, lNum, ;
         aArr := {}

   Default cSeparator := "", ;
           cType := "C"

   If ValType( cSeparator ) != "N" .and. cSeparator >= Chr( 32 )

      nEle := 1

      While ! Empty( cToken := StrToken( cString, nEle++, cSeparator ) )

         If cType == "N"
            AAdd( aArr, Val( cToken ) )
         ElseIf cType == "D"
            AAdd( aArr, CtoD( cToken ) )
         ElseIf cType == "L"
            AAdd( aArr, "T" $ cToken )
         Else
            AAdd( aArr, cToken )
         EndIf

      EndDo

   Else

      nEle := 1
      lNum := ValType( cSeparator ) == "N"

      While nEle <= Len( cString )

         Do Case
            Case cType == "N" .and. lNum
               AAdd( aArr, Val( SubStr( cString, nEle, cSeparator ) ) )
               nEle += cSeparator

            Case cType == "N" .and. ! lNum .and. cSeparator == ""
               AAdd( aArr, Val( SubStr( cString, nEle++, 1 ) ) )

            Case cType == "B" .and. lNum .and. cSeparator == 2  // "B" means Binary (not Block)
               AAdd( aArr, Bin2I( SubStr( cString, nEle, cSeparator ) ) )
               nEle += cSeparator

            Case cType == "B" .and. lNum .and. cSeparator == 4
               AAdd( aArr, Bin2L( SubStr( cString, nEle, cSeparator ) ) )
               nEle += cSeparator

            Case cType == "B" .and. lNum .and. cSeparator == 8
               AAdd( aArr, Bin2D( SubStr( cString, nEle, cSeparator ) ) )
               nEle += cSeparator

            Case cType == "C" .and. lNum
               AAdd( aArr, SubStr( cString, nEle, cSeparator ) )
               nEle += cSeparator

            Case cType == "D" .and. lNum
               AAdd( aArr, StoD( SubStr( cString, nEle, 8 ) ) )
               nEle += 8

            Case cType == "D" .and. ! lNum .and. cSeparator == ""
               AAdd( aArr, CtoD( SubStr( cString, nEle, 8 ) ) )
               nEle += 8

            Case cType == "L"
               AAdd( aArr, SubStr( cString, nEle++, 1 ) $ "T1" )

            Case cType == "A"       // "A" means Ascii (not Array)
               AAdd( aArr, Asc( SubStr( cString, nEle++, 1 ) ) )

            OtherWise
               AAdd( aArr, SubStr( cString, nEle++, 1 ) )

         EndCase

      EndDo

   EndIf

   If bBlock != Nil

      For nEle := 1 To Len( aArr )
         aArr[ nEle ] := Eval( bBlock, aArr[ nEle ] )
      Next

   EndIf

Return aArr

//--------------------------------------------------------------------------//
// creates a list string with or without separators from an array
// It's part of my personal Udf's collection.
// It's not used by Debug.prg but I incude it as a complement of CtoA function

Static Function AtoC( aArr, cType, cSep )

   Local nEle := 1, ;
         cStr := ""

   Default cType := ValType( aArr[ 1 ] ), ;
           cSep  := ""

   If ValType( cSep ) != "N"

      For nEle := 1 To Len( aArr )

         If cType == "D"
            cStr += ( DtoC( aArr[ nEle ] ) + cSep )
         ElseIf cType == "N"
            cStr += ( LTStr( aArr[ nEle ] ) + cSep )
         ElseIf cType == "L"
            cStr += ( If( aArr[ nEle ], "T", "F" ) + cSep )
         ElseIf cType == "A"    // "A" means Ascii (not Array)
            cStr += ( Chr( aArr[ nEle ] ) + cSep )
         Else
            cStr += ( aArr[ nEle ] + cSep )
         EndIf

      Next

   Else  // numeric "cSep" means fixed lenght elements with no separators

      For nEle := 1 To Len( aArr )

         If cType == "D"
            cStr += DtoS( aArr[ nEle ] )
         ElseIf cType == "N"
            cStr += Str( aArr[ nEle ], cSep )
         ElseIf cType == "L"
            cStr += If( aArr[ nEle ], "1", "0" )
         ElseIf cType == "B" .and. cSep == 2   // "B" means Binary (not Block)
            cStr += I2Bin( aArr[ nEle ] )
         ElseIf cType == "B" .and. cSep == 4
            cStr += L2Bin( aArr[ nEle ] )
         ElseIf cType == "B" .and. cSep == 8
            cStr += D2Bin( aArr[ nEle ] )
         Else
            cStr += PadR( aArr[ nEle ], cSep )
         EndIf

      Next

   EndIf

Return cStr

* =====================================================================================
* fShowDebug function shows the debug file at run time when uCtrlKey is pressed
*
* To make an external Debug.log file viewer, copy this part to a .prg file,
* (eg. DView.prg), uncomment the commented statements, compile with /n
* and build the executable.
*
* The usage from the command line could be (for example): DView c:\MyApp\Debug.log
* =====================================================================================

/*
#define _DDE_CH
#define _FOLDER_CH
#define _VIDEO_CH
#define _TREE_CH
#define _ODBC_CH
#define _OBJECTS_CH

#include "FiveWin.ch"
#include "TSBrowse.ch"

#define END_PROC_NAME  Chr(5)+Chr(32)

Static nMyHotKey
*/

Function fShowDebug( /*cMyDebugFile*/ )

   Local oWnd, oFont, oBrw, oMenu, cText, cLine, nLine, cProc, nEle, ;
         cVal, bClear, ;
         nWidth := 0, ;
         aArr   := {}, ;
         lClear := .F.

   If cMyDebugFile == Nil .or. ! File( cMyDebugFile )
      Return Nil
   EndIf

   bClear := &( "{||" + If( nMyHotKey == Nil, 'FErase("' + ;
                cMyDebugFile + '")', "SetDebug()" ) + "}" )

   DEFINE FONT oFont NAME "MS Sans Serif" SIZE 0, -11

   cText := MemoRead( cMyDebugFile )

   For nEle := 1 To MLCount( cText, 254 )

      cLine := MemoLine( cText, 254, nEle )

      If At( END_PROC_NAME, cLine ) > 0
         cProc := SubStr( cLine, 1, At( Chr( 5 ), cLine ) - 1 )
         nLine := Val( SubStr( cLine, At( Chr( 5 ), cLine ) + 2, ;
                               At( Chr( 7 ), cLine ) - ;
                               At( Chr( 5 ), cLine ) - 2 ) )
         cVal  := AllTrim( StrTran( SubStr( cLine, At( Chr( 7 ), ;
                                            cLine ) + 3 ), CRLF ) )
      Else
         cProc := nLine := ""
         cVal  := AllTrim( StrTran( cLine, CRLF ) )
      EndIf

      AAdd( aArr, { cProc, nLine, cVal } )

   Next

   If Empty( aArr )
      aArr := { { "", "", "" } }
   Else
      AEval( aArr, {|e| nWidth := Max( Len( e[1] ), nWidth ) } )
   EndIf

   MENU oMenu
      MENUITEM "&Clear" ACTION ( lClear := .T., ;
                                oBrw:SetArray( { { "", "", "" } } ), ;
                                oBrw:Reset() )
      MENUITEM "&Exit" ACTION oWnd:End()
   ENDMENU

   DEFINE WINDOW oWnd FROM 100, 100 TO 500, 640 PIXEL TITLE cMyDebugFile ;
          MENU oMenu

   nWidth := GetTextWidth( 0, Replicate( "B", Max( 1, nWidth ) ), oFont:hFont )
   nWidth := Max( 180, nWidth )

   @  0,  0 BROWSE oBrw ARRAY aArr OF oWnd SIZE 440, 300 CELLED FONT oFont ;
            COLORS CLR_BLUE, nRGB( 250, 250, 210 ), CLR_WHITE, CLR_CYAN, ;
                   CLR_BLUE, -1

   ADD COLUMN TO oBrw DATA ARRAY ELM 1 TITLE "Procedure" SIZE nWidth ;
       ALIGN DT_LEFT, DT_CENTER LOOK3D FALSE, TRUE

   ADD COLUMN TO oBrw DATA ARRAY ELM 2 TITLE "Line" SIZE 40 ;
       ALIGN DT_RIGHT, DT_CENTER LOOK3D FALSE, TRUE

   ADD COLUMN TO oBrw DATA ARRAY ELM 3 TITLE "Text" SIZE 400 ;
       ALIGN DT_LEFT, DT_CENTER LOOK3D FALSE, TRUE

   oBrw:nHeightHead += 4
   oBrw:nHeightHead += 2
   oWnd:oClient := oBrw

   ACTIVATE WINDOW oWnd ;
            ON INIT If( nMyHotKey == Nil, oWnd:Maximize(), Nil ) ;
            VALID ( oFont:End(), If( lClear, Eval( bClear ), Nil ), .T. )

Return Nil
